package info.bukova.isspst.services.requirement;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.Requirement;
import info.bukova.isspst.data.RequirementItem;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.RequirementSubject;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workflow;
import info.bukova.isspst.services.LazyLoader;
import info.bukova.isspst.services.approved.OrderService;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

public class RequirementServiceImpl extends
		RequirementBaseServiceImpl<Requirement> implements RequirementService,
		RequirementBaseService<Requirement> {

	@Autowired
	private RequirementTypeService reqTypeService;
	@Autowired
	private OrderService orderService;

	@Override
	protected Requirement createEntity() {
		Requirement entity = new Requirement();

		entity.setReqDate(new Date());
		entity.setType(reqTypeService.getTypeById(Constants.REQTYPE_ORDER));
		entity.setState(RequirementState.NEW);

		return entity;
	}

	@Override
	protected boolean autoApprove(Requirement entity) {
		List<User> approvers = this.getNextApprover(entity);
		Workflow nextWf = this.getNextWorkflow(entity);
		
		if (approvers == null || approvers.isEmpty() || nextWf == null || nextWf.getLimit() == null) {
			return false;
		}
		
		if (entity.getSumTotal().compareTo(nextWf.getLimit()) == -1) {
			approve(entity, approvers.get(0));
			return true;
		}
		
		return false;
	}
	
	@Override
	protected boolean canAdd(Requirement entity) {
		if (entity.getWorkgroup() != null 
				&& entity.getWorkgroup().getLimit() != null 
				&& entity.getWorkgroup().getLimit().compareTo(BigDecimal.ZERO) != 0) {
			
			BigDecimal total = orderService.totalOrderedForWorkgroup(entity.getWorkgroup());
			
			if (total == null) {
				total = BigDecimal.ZERO;
			}
			
			total = total.add(entity.getSumTotal());
			
			if (total.compareTo(entity.getWorkgroup().getLimit()) <= 0) {
				return true;
			} 
			
		} else {
			return true;
		}
		
		throw new AddRequirementException("LimitExceeded");
	}

	/*
	 * Lazy load pro načtení seznamu skupin materiálu nebo služeb do comba.
	 * Ošetřuje se zde případné neexistující vazby na ID smazaných skupin
	 * materiálu nebo služeb.
	 * 
	 * @see
	 * info.bukova.isspst.services.requirement.RequirementService#loadGroups
	 * (info.bukova.isspst.data.Requirement)
	 */
	@Transactional
	@LazyLoader("form")
	@Override
	public void loadGroups(Requirement req)
	{
		if (req == null)
		{
			return;
		}
		
		Requirement reqDb = this.getById(req.getId());
		
		// Načíst položky požadavku
		List<RequirementItem> items = reqDb.getItems();

		for (int i = 0; i < items.size(); i++)
		{
			RequirementItem item = items.get(i);

			try
			{
				// K položce získat skupinu materiálnu nebo služby
				RequirementSubject subject = item.getReqSubject();
				// Pokud ID skupiny nebylo nalezeno, padá do výjimky
				// Pokud ID skupiny bylo nalezeno, přinutí zavolání metody
				// načíst záznam se skupinou = lazy load
				subject.getCode();
				// Zapsat načtená data o skupině
				item.setReqSubject(subject);
			}
			catch (Throwable e)
			{
				item.setReqSubject(null);
			}
		}

		// Zaktualizovat položky záznamu tak, aby byly odkazy na skupiny
		// materiálů a služeb konzistentní
		req.setItems(items);
	}
}
